#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
#   calcReactionForce.py
#
#       指定したnodeGroupの反力を計算
#
#   16/08/17    新規作成
#

import sys
import glob
import pyFistr

def readFileLines(fileName):
    f = open(fileName); lines = f.readlines(); f.close()
    return lines

def getNodeNosInNodeGroup(grpName, lines):
    nodeNos = []
    i = pyFistr.getNextHeaderName(lines, 0, "!NGROUP", "NGRP="+grpName)
    if i < len(lines):
        i += 1
        [nodeNos, i] = pyFistr.getOnlyDataInLines(lines, i)
    return nodeNos

#
#  getNodeReactionForce
#    nodeNoに対応する反力を取得
def getNodeReactionForce(resFile, nodeNos):
    #最大nodeNoを取得
    nodes = []
    for nodeNo in nodeNos:
        nodes.append(int(nodeNo))
    nNode = max(nodes)
    #結果fileの読み込み
    lines = readFileLines(resFile)
    #各変数のcomponent数を取得
    i = 3
    comps = lines[i].split()
    nVal = len(comps)
    nFloat = 0
    for comp in comps:
        nFloat += int(comp)
    #各変数名を取得
    i += 1
    ip = i
    valNames = []
    while i < ip+nVal:
        line = lines[i]
        valNames.append(line.split()[0])
        i += 1
    ip = i
    #各変数の配列を準備
    for ii in range(len(valNames)):
        valNames[ii] = [valNames[ii], []]
    #各変数の値を取得
    m = 0
    while i < len(lines) and m < nNode:
        [nums, i] = get1data(lines, i, nFloat)
        n = 0
        for ii in range(len(comps)):
            comp = comps[ii]
            vals = nums[n:n+int(comp)]
            valNames[ii][1].append(vals)
            n += int(comp)
        m += 1
    #反力のみ取得
    reacs = []
    for valName in valNames:
        if valName[0] == "REACTION_FORCE":
            m = 1
            for val in valName[1]:
                if isInGroup(m, nodes) == True:
                    reacs.append(val)
                m += 1
    #合力を取得
    tReac = [0,0,0]
    for reac in reacs:
        tReac[0] += float(reac[0])
        tReac[1] += float(reac[1])
        tReac[2] += float(reac[2])
    return tReac

#  isInGroup
#    group内のnodeNoかどうかcheck
def isInGroup(node, nodes):
    ans = False
    for nodeNo in nodes:
        if node == nodeNo:
            ans = True
            break
    return ans


#  get1data
#    1行分のデータを取得
def get1data(lines, i, nFloat):
    nums = []
    while len(nums) < nFloat+1 and i < len(lines):
        nums += lines[i].split()
        i += 1
    return [nums[1:], i]    

#
#  getReactionForce
#    反力を取得
def getReactionForce(nodeNos):
    files = glob.glob("FistrModel.res.*")
    stepFiles = []
    for name in files:
        step = int(name.split(".")[-1])
        stepFiles.append([step, name])
    stepFiles.sort()
    resFiles = []
    for name in stepFiles:
        resFiles.append(name[1])
    #reaction = getNodeReactionForce(resFiles[0], nodeNos)
    for resFile in resFiles:
        reaction = getNodeReactionForce(resFile, nodeNos)
        print "----", (reaction[0]**2 + reaction[1]**2 + reaction[2]**2)**0.5, reaction[0]


#
#  getNodeNos
#    nodeGroupのnodeNoを取得
def getNodeNos(grpName):
    lines = readFileLines("FistrModel.msh")
    grpNodes = getNodeNosInNodeGroup(grpName, lines)
    forces = getReactionForce(grpNodes)


if __name__=="__main__":
    nodeGroup = sys.argv[1]
    nodeNos = getNodeNos(nodeGroup)


