#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   keepFaceShape.py
#
#       面の形状を維持する様に、「!EQUATION」を追加する。
#
#   16/11/13    新規作成


import sys
import pyFistr


#  addVector
#    ベクトルの加算
def addVector(a, b):
    x = a[0] + b[0]
    y = a[1] + b[1]
    z = a[2] + b[2]
    return [x,y,z]

#  crossProduct
#    ベクトル積（外積）
def crossProduct(a, b):
    x = a[1]*b[2] - a[2]*b[1]
    y = a[2]*b[0] - a[0]*b[2]
    z = a[0]*b[1] - a[1]*b[0]
    return [x, y, z]

#  dotProduct
#    スカラー積（内積）
def dotProduct(a, b):
    ans = a[0]*b[0] + a[1]*b[1] + a[2]*b[2]
    return ans

#  mag
#    vectorの絶対値の取得
def mag(a):
    ans = pow((a[0]*a[0] + a[1]*a[1] + a[2]*a[2]), 0.5)
    return ans

#  getLength
#    2点間の距離を算出
def getLength(a, b):
    vec = getVector(a, b)
    l = mag(vec)
    return l

#  getVector
#    2点間のベクトルを作成
def getVector(a, b):
    x = b[0] - a[0]
    y = b[1] - a[1]
    z = b[2] - a[2]
    return [x,y,z]

#  normalVector
#    単位ベクトルを取得
def normalVector(vec):
    l = mag(vec)
    x = vec[0]/l
    y = vec[1]/l
    z = vec[2]/l
    return [x,y,z]

#  readFileLines
#    fileの読み込み
def readFileLines(fileName):
    f = open(fileName)
    lines = f.readlines()
    f.close()
    return lines

#  writeFileLines
#    fileの書き込み
def writeFileLines(fileName, lines):
    f = open(fileName, "w")
    for line in lines:
        f.write(line)
    f.close()

#
#  getKeepFaceNodeNo
#    faceのnodeNoを取得
def getKeepFaceNodeNo(lines, keepFace):
    #header行を取得
    i = pyFistr.getNextHeaderName(lines, 0, "!NGROUP", "NGRP="+keepFace)
    if i >= len(lines):
        return []

    sp = i + 1
    [nodes, i] = pyFistr.getOnlyDataInLines(lines, sp)
    #数字に変換
    nodeNos = []
    for node in nodes:
        nodeNos.append(int(node))
    return nodeNos

#
#  getDataLines
#    data行を取得
def getDataLines(lines, i):
    getLines = []
    while i < len(lines):
        line = lines[i]
        if not (line[:2] == "!!" or line[0] == "#"):
            if line[0] == "!":
                break
            else:
                getLines.append(line)
        i += 1
    return [getLines, i]

#
#  getKeepFaceLocation
#    faceの座標を取得
def getKeepFaceLocation(lines, faceNodes):
    nodeLines = []
    i = 0
    while i < len(lines):
        i = pyFistr.getNextHeaderName(lines, i, "!NODE", "")
        [getLines, i] = getDataLines(lines, i+1)
        nodeLines += getLines
    newLines = []
    for line in nodeLines:
        words = pyFistr.deleteSp(line).split(",")
        nodeNo = int(words[0])
        for faceNode in faceNodes:
            if nodeNo == faceNode:
                newLines.append(words)
                break
        if len(newLines) == len(faceNodes):
            break
    faceLocs = []
    for words in newLines:
        loc = []
        for word in words[1:]:
            loc.append(float(word))
        faceLocs.append(loc)
    return faceLocs

#
#  getFaceCenterPoint
#    faceの中心座標を取得
def getFaceCenterPoint(faceLocs):
    x = 0.0; y = 0.0; z = 0.0
    for faceLoc in faceLocs:
        x += faceLoc[0]
        y += faceLoc[1]
        z += faceLoc[2]
    centerPoint = [x/len(faceLocs), y/len(faceLocs), z/len(faceLocs)]
    return centerPoint

#
#  getNormalVectorEachAxis
#    各軸回転方向の単位ベクトルを取得
def getNormalVectorEachAxis(faceLocs, centerPoint):
    nVecs = []
    for faceLoc in faceLocs:
        lx = faceLoc[0] - centerPoint[0]
        ly = faceLoc[1] - centerPoint[1]
        lz = faceLoc[2] - centerPoint[2]
        #x軸（yz平面）
        vecYZ = [0, ly, lz]
        vecRx = crossProduct(vecYZ, [1, 0, 0])
        nVecRx = normalVector(vecRx)
        #y軸（zx平面）
        vecZX = [lx, 0, lz]
        vecRy = crossProduct(vecZX, [0,1,0])
        nVecRy = normalVector(vecRy)
        #z軸（xy平面）
        vecXY = [lx, ly, 0]
        vecRz = crossProduct(vecXY, [0,0,1])
        nVecRz = normalVector(vecRz)
        #保存
        nVec = [nVecRx, nVecRy, nVecRz]
        nVecs.append(nVec)
    return nVecs

#
#  getRotationRadius
#    各方向の回転半径を取得
def getRotationRadius(faceLocs, centerPoint):
    Rs = []
    for faceLoc in faceLocs:
        lx = faceLoc[0] - centerPoint[0]
        ly = faceLoc[1] - centerPoint[1]
        lz = faceLoc[2] - centerPoint[2]
        vec = [lx, ly, lz]
        rx = dotProduct(vec, [1,0,0])
        ry = dotProduct(vec, [0,1,0])
        rz = dotProduct(vec, [0,0,1])
        #X軸回転（yz平面上の半径）
        vecx = [0, ry, rz]
        #vecx = addVector(ry, rz)
        Rx = getLength(vecx, [0,0,0])
        #Rx = pow((ry*ry + rz*rz), 0.5)
        #Y軸回転（xz平面上の半径）
        vecy = [rx, 0, rz]
        #vecy = addVector(rx, rz)
        Ry = getLength(vecy, [0,0,0])
        #Ry = pow((rx*rx + rz*rz), 0.5)
        #Z軸回転（xy平面上の半径）
        vecz = [rx, ry, 0]
        #vecz = addVector(rx, ry)
        Rz = getLength(vecz, [0,0,0])
        #Rz = pow((rx*rx + ry*ry), 0.5)
        Rs.append([Rx, Ry, Rz])
    return Rs

#
#  getEquationLines
#    !EQUATIONの行を作成
def getEquationLines(faceNodes, nVecs, Rxyzs):

    def create1Equation(nodeNo, vec, r):
        No = str(nodeNo)
        cx = str(vec[0]/r)
        cy = str(vec[1]/r)
        cz = str(vec[2]/r)
        list1 = [No, "1", cx] + [No, "2", cy] + [No, "3", cz]
        line1 = ", ".join(list1)
        return line1        

    def createEquation(node1, vec1, r1, node2, vec2, r2):
        line1 = create1Equation(node1, vec1, r1)
        line2 = create1Equation(node2, vec2, r2)
        lines = ["6, 0.0\n"]
        lines += [line1 + ",  " + line2 + "\n"]
        return lines

    newLines = []
    i = 0
    while i < len(faceNodes)-1:
        nodeNo1 = faceNodes[0]
        nodeNo2 = faceNodes[i+1]
        #X軸回転
        r1 =   Rxyzs[0][0]
        vec1 = nVecs[0][0]
        r2 =   Rxyzs[i+1][0]
        vec2 = nVecs[i+1][0]
        lines = createEquation(nodeNo1, vec1, r1, nodeNo2, vec2, r2)
        newLines += lines
        """
        #Y軸回転
        r1 =   Rxyzs[0][1]
        vec1 = nVecs[0][1]
        r2 =   Rxyzs[i+1][1]
        vec2 = nVecs[i+1][1]
        lines = createEquation(nodeNo1, vec1, r1, nodeNo2, vec2, r2)
        newLines += lines
        #Z軸回転
        r1 =   Rxyzs[0][2]
        vec1 = nVecs[0][2]
        r2 =   Rxyzs[i+1][2]
        vec2 = nVecs[i+1][2]
        lines = createEquation(nodeNo1, vec1, r1, nodeNo2, vec2, r2)
        newLines += lines
        """
        i += 1
    header = ["!EQUATION\n"]
    newLines = header + newLines
    return newLines

#
#  addEquationLines
#    !EQUATIONを挿入
def addEquationLines(lines, eqLines):
    #挿入場所を検索
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!ELEMENT")] == "!ELEMENT":
            break
        i += 1
    #コメント文を遡る
    i -= 1
    while i > 0:
        line = lines[i]
        if not (line[0] == "#" or line[:2] == "!!"):
            break
        i -= 1
    #挿入場所に挿入する
    isp = i + 1
    newLines = lines[:isp] + eqLines + lines[isp:]
    return newLines

#
#  keepFaceShape
#    面の形状を維持させる
def keepFaceShape(mshFile, keepFace):
    lines = readFileLines(mshFile)
    faceNodes = getKeepFaceNodeNo(lines, keepFace)
    faceLocs = getKeepFaceLocation(lines, faceNodes)
    centerPoint = getFaceCenterPoint(faceLocs)
    #角軸回転単位ベクトルの取得
    nVecs = getNormalVectorEachAxis(faceLocs, centerPoint)
    #回転半径を取得
    Rs = getRotationRadius(faceLocs, centerPoint)
    #EQUATIONを作成
    eqLines = getEquationLines(faceNodes, nVecs, Rs)
    #EQUATIONを追加
    newLines = addEquationLines(lines, eqLines)
    writeFileLines("FistrModel.msh", newLines)


if __name__=='__main__':
    #mshFile = sys.argv[1]
    #keepFace = sys.argv[2]
    mshFile = "FistrModel-org.msh"
    keepFace = "beamConnect"
    keepFaceShape(mshFile, keepFace)
