#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   unv2fistr.py
#
#       unv →　abaqus　→　fistr　の順にメッシュ変換
#
#       unv2fistr.py <unvファイル名>
#
#
#   15/03/19    新規作成
#      09/01    windows対応のため、os.sepに修正
#      10/27    unvファイルの行末修正（file内容チェック）を追加
#   16/06/15    ヘルプ表示を修正
#   17/09/26    高速化の為、fileCheck時の書き出し方法を修正
#

import sys
import os
import glob

#
#  printMsg
#
def printMsg():
    msg = """
<unv2fistr.py>
unvフォーマットのmeshをFrontISTR用mesh(HECMWフォーマット）に変換する。
この変換は、unv -> abaqus -> fistr の順番で変換している。

<使い方>
unv2fistr.py <unvファイル>

例：
unv2fistr.py name.unv
unv2fistr.py name
"""
    print msg

#
#  abaqus2fistr
#    abaqusをfistrに変換
def abaqus2fistr(fileName, newName):
    print
    print "convert abaqus to fistr..."
    comm = "abaqus2fistr.py " + fileName
    print comm
    os.system(comm)

#
#  unv2abaqus
#    unvをabaqusに変換
def unv2abaqus(fileName, newName):
    print
    print "convert unv to abaqus..."
    comm = "unv2abaqusEx.py " + fileName + " " + newName + " N"
    print comm
    os.system(comm)


if __name__=='__main__':
    if len(sys.argv) == 1:
        printMsg()
        exit()

    name = sys.argv[1]
    if name[:len("-h")] == "-h" or name[:len("--help")] == "--help":
        printMsg()
    else:
        #file名を取得
        fileDir = os.sep.join(sys.argv[1].split(os.sep)[:-1])
        if fileDir == "":
            fileDir = os.getcwd()
        fileName = sys.argv[1].split(os.sep)[-1]
        if fileName[-4:] != ".unv":
            fileName += ".unv"
        #file有無チェック
        if len(glob.glob(fileDir+os.sep+fileName)) == 0:
            print
            print "error:「" + fileDir + os.sep + fileName + "」が存在しません。"
            exit()
        #file内容をチェック
        print 
        print "checking file contents..."
        f=open(fileDir+os.sep+fileName); lines=f.readlines(); f.close()
        w=open(fileDir+os.sep+fileName, "w")
        cont = "".join(lines)
        w.write(cont)
        w.close()
        lines = []
        #for line in lines:
        #    w.write(line)
        #w.close()
        #abaqusに変換
        newName = fileName.split(".")[0]
        unv2abaqus(fileDir+os.sep+fileName, fileDir+os.sep+newName)
        #fistrに変換
        abaqusName = newName + ".inp"
        fistrName = newName + ".msh"
        abaqus2fistr(fileDir+os.sep+abaqusName, fileDir+os.sep+fistrName)

