#!/usr/bin/env python  
# -*- coding: utf-8 -*-    
#
#       addOutputItemsDialog.py
#
#           出力項目を追加する
#
#   15/12/18    新規作成
#      12/25    行末コードlsを修正
#   16/01/06    tresca応力ひずみを追加
#      03/29    応力、ひずみtensorが選択できるように修正
#      04/05    バグ修正
#      04/07    vtkファイルの抽出方法を修正
#
#

import sys   
import os   
import os.path   
import pygtk
import gtk
import glob
import pyFistr

#ls = os.linesep
ls = "\n"
outItems = []
nodalStress = []        #応力tensor名
nodalStrain = []        #ひずみtensor名


def getOutputFlag():
    windowApp()  
    gtk.main()
    return outItems


class windowApp:  
    def __init__(self):

        #windows版のpangoWarning対策で以下の3行を追加。
        #　easyIstr_1.gladeを読み込む処理を追加
        gladefile = "easyIstr_1.glade"  
        self.builder = gtk.Builder()  
        self.builder.add_from_file(os.getenv("easyIstrPath") + os.sep + gladefile)

        #Set the Glade file
        gladefile = "addOutputItemsDialog_1.glade"  
        self.builder = gtk.Builder()  
        self.builder.add_from_file(os.path.dirname(os.path.abspath(__file__)) +
                os.sep + gladefile)  

        self.setGtkObject()
        self.subInitialize()

        #windowを表示
        self.builder.connect_signals(self)
        self.mainWindow = self.builder.get_object ("window3")
        title = _(u"出力項目を追加")
        self.mainWindow.set_title(title)
        self.mainWindow.show_all()
        self.startingErrorCheck()

    #
    #  setGtkObject
    #    objectの定義
    def setGtkObject(self):
        #comboのmodelを定義
        self.listmodelStressTensor = gtk.ListStore(str)
        self.listmodelStrainTensor = gtk.ListStore(str)
        #comboBox
        self.comboBox_stressTensor = self.builder.get_object("combobox1")
        self.comboBox_strainTensor = self.builder.get_object("combobox2")
        #checkBtn
        self.checkBtn_misesStress = self.builder.get_object("checkbutton1")
        self.checkBtn_sigma = self.builder.get_object("checkbutton2")
        self.checkBtn_sigmaVec = self.builder.get_object("checkbutton3")
        self.checkBtn_misesStrain = self.builder.get_object("checkbutton4")
        self.checkBtn_epsilon = self.builder.get_object("checkbutton5")
        self.checkBtn_epsilonVec = self.builder.get_object("checkbutton6")
        self.checkBtn_trescaStress = self.builder.get_object("checkbutton7")
        self.checkBtn_trescaStrain = self.builder.get_object("checkbutton8")

    #
    #  subInitialize
    #    初期化
    def subInitialize(self):
        global nodalStress, nodalStrain
        [nodalStress, nodalStrain] = readTensorsInVtk()
        [addFlag, stressTensor, strainTensor] = readAddFlag()
        self.setCheckBtn(addFlag)
        self.setStressToComboBox()
        self.setStrainToComboBox()
        self.selectTensor(stressTensor, strainTensor)
        return

    #追加
    def onAddItems(self, event):
        outItems = self.addItems(event)
        [stressTensor, strainTensor] = self.getSelectTensor()
        #エラーチェック
        if self.isStressStrainTensors(stressTensor, strainTensor, outItems) == False:
            return
        #出力項目追加
        saveAddFlag(outItems, stressTensor, strainTensor)
        flag = " ".join(outItems)
        comm = "addPrincipalStressWithDialog.py " + flag + " " + stressTensor + " " + strainTensor
        pyFistr.runAppBG(comm)
        gtk.main_quit()

    #キャンセル
    def onCancel(self, event):
        gtk.main_quit()

    #
    #  startingErrorCheck
    #    起動時のエラーチェック
    def startingErrorCheck(self):
        title = u"エラー"
        mess = ""
        files = getVtkFileNames()
        if len(files) == 0:
            mess = u"vtk変換したファイルが存在しません。" + ls
            mess += u"結果ファイルをデータ変換してください。"
            self.errDialog(title, mess)
            return False

    #
    #  isStressStrainTensors
    #    vtkファイル内に指定したtensorがあるかどうかチェック
    def isStressStrainTensors(self, stressTensor, strainTensor, outItems):
        n = 0
        for flag in outItems:
            if flag == "1":
                n += 1
        if n == 0:
            title = u"エラー"
            mess = u"出力項目が選択されていません。" + ls
            mess += u"出力項目を選択してください。"
            self.errDialog(title, mess)
            return False

        #応力項目を追加
        if (outItems[0] == "1" or outItems[1] == "1"
            or outItems[2] == "1" or outItems[6] == "1"):
            if stressTensor == "none":
                title = u"エラー"
                mess = u"結果ファイルに NodalSTRESS が無い為、応力側は選択できません。" + ls
                mess += u"応力側を選択する場合は、" + ls
                mess += u"節点応力（NSTRESS）を出力項目に追加して、solverを再実行する必要があります。"
                self.errDialog(title, mess)
                return False

        #ひずみ項目を追加
        if (outItems[3] == "1" or outItems[4] == "1"
            or outItems[5] == "1" or outItems[7] == "1"):
            if strainTensor == "none":
                title = u"エラー"
                mess = u"結果ファイルに NodalSTRAIN が無い為、ひずみ側は選択できません。" + ls
                mess += u"ひずみを選択する場合は、" + ls
                mess += u"節点ひずみ（NSTRAIN）を出力項目に追加して、solverを再実行する必要があります。"
                self.errDialog(title, mess)
                return False
        return True
            
    #
    #  getSelectTensor
    #
    def getSelectTensor(self):
        n = self.comboBox_stressTensor.get_active()
        if n < 0:
            stress = "none"
        else:
            stress = nodalStress[n]
        n = self.comboBox_strainTensor.get_active()
        if n < 0:
            strain = "none"
        else:
            strain = nodalStrain[n]
        return [stress, strain]

    #
    #  selectTensor
    #
    def selectTensor(self, stressTensor, strainTensor):
        flag = 0
        for i in range(len(nodalStress)):
            if nodalStress[i] == stressTensor:
                self.comboBox_stressTensor.set_active(i)
                flag = 1
                break
        if flag == 0:
            self.comboBox_stressTensor.set_active(0)
        flag = 0
        for i in range(len(nodalStrain)):
            if nodalStrain[i] == strainTensor:
                self.comboBox_strainTensor.set_active(i)
                flag = 1
                break
        if flag == 0:
                self.comboBox_strainTensor.set_active(0)

    #
    #  setStressToComboBox
    #    応力tensor名をcomboBoxにセット
    def setStressToComboBox(self):
        self.listmodelStressTensor.clear()
        self.comboBox_stressTensor.clear()
        for tensor in nodalStress:
            self.listmodelStressTensor.append([tensor])
        cell = gtk.CellRendererText()
        self.comboBox_stressTensor.pack_start(cell, True)
        self.comboBox_stressTensor.add_attribute(cell, "text", 0)
        self.comboBox_stressTensor.set_model(self.listmodelStressTensor)

    #
    #  setStrainToComboBox
    #    ひずみtensor名をcomboBoxにセット
    def setStrainToComboBox(self):
        self.listmodelStrainTensor.clear()
        self.comboBox_strainTensor.clear()
        for tensor in nodalStrain:
            self.listmodelStrainTensor.append([tensor])
        cell = gtk.CellRendererText()
        self.comboBox_strainTensor.pack_start(cell, True)
        self.comboBox_strainTensor.add_attribute(cell, "text", 0)
        self.comboBox_strainTensor.set_model(self.listmodelStrainTensor)

    #
    #  addItems
    #    出力項目を読み取り、項目を追加する。
    def addItems(self, event):

        def getFlag(flag):
            ans = "0"
            if flag == True:
                ans = "1"
            return ans

        outItems = []
        outItems.append(getFlag(self.checkBtn_misesStress.get_active()))
        outItems.append(getFlag(self.checkBtn_sigma.get_active()))
        outItems.append(getFlag(self.checkBtn_sigmaVec.get_active()))
        outItems.append(getFlag(self.checkBtn_misesStrain.get_active()))
        outItems.append(getFlag(self.checkBtn_epsilon.get_active()))
        outItems.append(getFlag(self.checkBtn_epsilonVec.get_active()))
        outItems.append(getFlag(self.checkBtn_trescaStress.get_active()))
        outItems.append(getFlag(self.checkBtn_trescaStrain.get_active()))
        return outItems

    #
    #  setCheckBtn
    #    checkBtnをセット
    def setCheckBtn(self, addFlag):
        #応力
        if addFlag[0] == "0":
            self.checkBtn_misesStress.set_active(False)
        else:
            self.checkBtn_misesStress.set_active(True)
        if addFlag[1] == "0":
            self.checkBtn_sigma.set_active(False)
        else:
            self.checkBtn_sigma.set_active(True)
        if addFlag[2] == "0":
            self.checkBtn_sigmaVec.set_active(False)
        else:
            self.checkBtn_sigmaVec.set_active(True)
        if addFlag[6] == "0":
            self.checkBtn_trescaStress.set_active(False)
        else:
            self.checkBtn_trescaStress.set_active(True)
        #ひずみ
        if addFlag[3] == "0":
            self.checkBtn_misesStrain.set_active(False)
        else:
            self.checkBtn_misesStrain.set_active(True)
        if addFlag[4] == "0":
            self.checkBtn_epsilon.set_active(False)
        else:
            self.checkBtn_epsilon.set_active(True)
        if addFlag[5] == "0":
            self.checkBtn_epsilonVec.set_active(False)
        else:
            self.checkBtn_epsilonVec.set_active(True)
        if addFlag[7] == "0":
            self.checkBtn_trescaStrain.set_active(False)
        else:
            self.checkBtn_trescaStrain.set_active(True)


    def okDialog(self, title, message):
        dlg = gtk.MessageDialog(type=gtk.MESSAGE_INFO, buttons=gtk.BUTTONS_OK)
        dlg.set_title(title)
        #messageを分ける
        mess1 = message.split(ls)[0]
        mess2 = ls.join(message.split(ls)[1:])
        dlg.set_markup(mess1)
        dlg.format_secondary_text(mess2)
        ans = dlg.run() 
        dlg.destroy()

    def errDialog(self, title, message):
        dlg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK)
        dlg.set_title(title)
        mess1 = message.split(ls)[0]
        mess2 = ls.join(message.split(ls)[1:])
        dlg.set_markup(mess1)
        dlg.format_secondary_text(mess2)
        ans = dlg.run() 
        dlg.destroy()


#
#  getVtkFileNames
#    vtkファイル名の取得
def getVtkFileNames():
    if len(glob.glob("hecmw_part_ctrl.dat")) == 0:
        para = "0"
    else:
        para = "merge"
    solType = pyFistr.getSolutionTypeFromCnt()
    if solType == "EIGEN":
        resFileHeader = "convFistrModel_eigen"
    elif solType == "HEAT" or solType == "HEAT_D":
        resFileHeader = "convFistrModel_temp"
    else:
        resFileHeader = "convFistrModel"
    filesCard = resFileHeader + ".res." + para + "*.vtk"
    files = glob.glob(filesCard)
    return files

#
#  readTensorsInVtk
#    vtkファイル内のtensor名を戻す
def readTensorsInVtk():
    stress = []; strain = []
    files = getVtkFileNames()
    if len(files) == 0:
        return [stress, strain]    

    files.sort()
    fileName = files[0]
    f=open(fileName); lines=f.readlines(); f.close()
    for line in lines:
        n = line.find("NodalSTRESS")
        if n >= 0:
            p = pyFistr.skipChara(line, n)
            word = line[n:p]
            stress.append(word)
        n = line.find("NodalSTRAIN")
        if n >= 0:
            p = pyFistr.skipChara(line, n)
            word = line[n:p]
            strain.append(word)
    return [stress, strain]

#
#  readAddFlag
#    追加する項目をfileから読み込み
def readAddFlag():
    fileName = os.getenv("easyIstrUserPath")+os.sep+"data"+os.sep+"easyIstr_data"
    addFlag = ["0", "1", "0", "0", "0", "0", "0", "0"]
    stressTensor = "NodalSTRESS"
    strainTensor = "NodalSTRAIN"
    if len(glob.glob(fileName)) != 0:
        f=open(fileName); lines=f.readlines(); f.close()
        for line in lines:
            words = line.split()
            if len(words) > 1:
                if words[0] == "addFlag":
                    for i in range(len(words[1:])):
                        flag = words[i+1]
                        addFlag[i] = flag
                elif words[0] == "stressTensor":
                    stressTensor = words[1]
                elif words[0] == "strainTensor":
                    strainTensor = words[1]
    return [addFlag, stressTensor, strainTensor]

#
#  saveAddFlag
#    addFlagを書き込み
def saveAddFlag(addFlag, stressTensor, strainTensor):
    addLine = "addFlag " + " ".join(addFlag)
    fileName = os.getenv("easyIstrUserPath")+os.sep+"data"+os.sep+"easyIstr_data"
    if len(glob.glob(fileName)) == 0:
        cont = "#addOutputItems" + ls + addLine + ls
        f=open(fileName, "w"); f.write(cont); f.close()
    else:
        f=open(fileName); lines=f.readlines(); f.close()
        flag = 0; stressF = 0; strainF = 0
        for i in range(len(lines)):
            line = lines[i]
            words = line.split()
            if len(words) > 1:
                if words[0] == "addFlag":
                    lines[i] = addLine + "\n"
                    flag = 1
                elif words[0] == "stressTensor":
                    lines[i] = "stressTensor " + stressTensor + "\n"
                    stressF = 1
                elif words[0] == "strainTensor":
                    lines[i] = "strainTensor " + strainTensor + "\n"
                    strainF = 1
        if flag == 0:
            lines += ["#addOutputItems\n"]
            lines += [addLine + "\n"]
        if stressF == 0:
            lines += ["stressTensor " + stressTensor + "\n"]
        if strainF == 0:
            lines += ["strainTensor " + strainTensor + "\n"]
        f=open(fileName, "w")
        for line in lines:
            f.write(line)
        f.close()


if __name__ == "__main__":  
    import gettext
    gettext.install("app") # replace with the appropriate catalog name

    windowApp()  
    gtk.main()
    
