#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
#
#       pyFistr.py
#
#   15/09/02    windows対応の為、"\n"→os.linesepに修正
#      10/30    deleteLsOfLineを追加
#      12/17    addPrincipalStressDataInVtk:主応力のベクトルを追加
#      12/18    addPrincipalStrainDataInVtk:主ひずみを追加
#      12/25    行末コードlsを修正
#   16/01/06    addPrincipalStressDataInVtk:tresca応力
#               addPrincipalStrainDataInVtk:trescaひずみを追加
#      01/16    getEndPoint:追加
#      03/27    addPrincipalStressDataInVtk:取得する物性値名を出力
#               addPrincipalStrainDataInVtk:取得する物性値名を出力
#      04/07    getSolutionTypeFromCnt:追加
#      04/17    changeMeshScale:修正
#   17/08/23    readUsingApplications:追加
#      09/23    高速化の為、meshをmeshHeaderDataとして保存して処理する様に修正
#

import abaqus2fistr
import calcPrincipalStress
import csv
import os
import glob
import math

#ls = os.linesep             #行末コード
ls = "\n"

#------ data 変換--------------------
#
#  getElementTypeFistr2inp
#    fistrの要素タイプをinp用に変換
def getElementTypeFistr2inp(nameType):
    data = [["341", "tet"], ["342", "tet2"], ["361", "hex"], ["362", "hex2"],
            ["351", "prism"], ["352", "prism2"]]
    for names in data:
        if names[0] == nameType:
            inpType = names[1]
    return inpType

#
#  getNodesOrderOfElement
#    faceNoからelementのnodeNoを取得
#    nodeNoの並びは、fistrの並び順
def getNodesOrderOfElement(elType, faceNo):
    data = [["tet", [[1,2,3], [1,2,4], [2,3,4], [1,3,4]]],
            ["tet2", [[1,2,3], [1,2,4], [2,3,4], [1,3,4]]],
            ["hex", [[1,2,3,4], [5,6,7,8], [1,2,5,6], [2,3,6,7], [3,4,7,8],
                     [1,4,5,8]]],
            ["hex2", [[1,2,3,4], [5,6,7,8], [1,2,5,6], [2,3,6,7], [3,4,7,8],
                     [1,4,5,8]]],
            ["prism", [[1,2,3], [4,5,6], [1,2,4,5], [2,3,5,6], [1,3,4,6]]],
            ["prism2", [[1,2,3], [4,5,6], [1,2,4,5], [2,3,5,6], [1,3,4,6]]]]
    nodes = []
    for typeFaceNo in data:
        if typeFaceNo[0] == elType:
            nodes = typeFaceNo[1][faceNo-1]
            break
    return nodes

#
#  changeMeshScale
#    meshのscaleを変更
def changeMeshScale(meshHeaderData, scale):
    for n in range(len(meshHeaderData)):
        line = meshHeaderData[n][0]
        if line[:len("!NODE")] == "!NODE":
            data = meshHeaderData[n][1]
            for i in range(len(data)):
                data[i][1] = data[i][1] * scale
                data[i][2] = data[i][2] * scale
                data[i][3] = data[i][3] * scale
            meshHeaderData[n][1] = data
    return meshHeaderData

#
#  getNodesFromElementSurface
#    elementとfaceNoからnodesを取得
def getNodesFromElementSurface(elements, elementNo, faceNo):
    elType = elements[elementNo][0]
    nodesAll = elements[elementNo][1:]
    nodesStd = getNodesOrderOfElement(elType, faceNo)
    nodes = []
    for node in nodesStd:
        nodes.append(nodesAll[node-1])
    return nodes

#
#  getSurfaceGroupsInLines
#    surfaceGroupを取得
def getSurfaceGroupsInLines(lines):
    surGrp = []
    i = 0
    while i < len(lines):
        i = getNextHeaderName(lines, i, "!SGROUP", "")
        if i < len(lines):
            i += 1
            [surs, i] = getOnlyDataInLines(lines, i)
            surGrp.append(surs)
    return surGrp

#
#  getNodeGroupsInLines
#    nodeGroupを取得
def getNodeGroupsInLines(lines):
    nodeGrp = []
    i = 0
    while i < len(lines):
        i = getNextHeaderName(lines, i, "!NGROUP", "")
        if i < len(lines):
            i += 1
            [nodes, i] = getOnlyDataInLines(lines, i)
            nodeGrp.append(nodes)
    return nodeGrp

#  getOnlyDataInLines
#    指定行からdataのみ取得
def getOnlyDataInLines(lines, i):
    data = []
    while i < len(lines):
        if lines[i][0] != "!":
            if lines[i][0] != "#":
                words = deleteSp(lines[i]).split(",")
                for word in words:
                    if word != "":
                        data += [word]
        else:
            break
        i += 1
    return [data, i]

#  deleteOnlyDataInLines
#    指定行からdataのみ削除
def deleteOnlyDataInLines(lines, i):
    ip = i
    while i < len(lines):
        if lines[i][0] == "!" or lines[i][0] == "#":
            break
        i += 1
    newLines = lines[:ip] + lines[i:]
    return [newLines, ip]

#
#  getMeshDataForParaview
#    paraview用のmeshData(nodeとelement）を取得
def getMeshDataForParaview(fileName):
    f=open(fileName); lines=f.readlines(); f.close()
    nNode = int(lines[3].split()[0])
    nElement = int(lines[3].split()[1])
    meshLines = lines[4:nNode+nElement+4]
    return [meshLines, nNode, nElement]

#
#  addMeshDataConvVisData
#    fistr用結果dataをvis用に変換
def addMeshDataConvVisData(lines, meshData):
    #結果データを成形
    newLine = []
    i = 1
    nNode = int(lines[i].split()[0])
    nElement = int(lines[i].split()[1])
    i += 1
    nNodeVal = int(lines[i].split()[0])
    nElementVal = int(lines[i].split()[1])
    i += 1
    #nodeのデータ取得
    nodeValName = lines[i:i+nNodeVal+1]
    words = lines[i].split()
    nValue = 0
    for word in words:
        nValue += int(word)
    i += nNodeVal + 1
    newNodeVal = []
    n = 0
    while n < nNode:
        No = lines[i].split()[0]
        line = No + " "
        i += 1
        nVal = 0
        while nVal < nValue:
            #subLine = lines[i].split(ls)[0]
            subLine = deleteLsOfLine(lines[i])
            line += subLine + " "
            nVal += len(subLine.split())
            i += 1
        newNodeVal.append(line + ls)
        n += 1
    #elementのデータを取得
    if i < len(lines):
        elementValName = lines[i:i+nElementVal+1]
        nValue = 0
        words = lines[i].split()
        for word in words:
            nValue += int(word)
        i += nElementVal + 1
        newElementVal = []
        n = 0
        while n < nElement:
            No = lines[i].split()[0]
            line = No + " "
            i += 1
            nVal = 0
            while nVal < nValue:
                #subLine = lines[i].split(ls)[0]
                subLine = deleteLsOfLine(lines[i])
                line += subLine + " "
                nVal += len(subLine.split())
                i += 1
            newElementVal.append(line + ls)
            n += 1
    else:
        elementValName = []
        newElementVal = []
    #変数名の修正
    nodeValName[0] = str(nNodeVal) + " " + " ".join(nodeValName[0].split()) + ls
    i = 0
    while i < nNodeVal:
        #nodeValName[i+1] = nodeValName[i+1].split(ls)[0] + ", unit_unknown" + ls
        nodeValName[i+1] = deleteLsOfLine(nodeValName[i+1]) + ", unit_unknown" + ls
        i += 1
    if len(elementValName) != 0:
        elementValName[0] = (str(nElementVal) + " " +
                " ".join(elementValName[0].split()) + ls)
        i = 0
        while i < nElementVal:
            #elementValName[i+1] = (elementValName[i+1].split(ls)[0] +
            #    ", unit_unknown" + ls)
            elementValName[i+1] = (deleteLsOfLine(elementValName[i+1]) +
                ", unit_unknown" + ls)
            i += 1
    #newData作成
    newLines = [" ".join([str(nNode), str(nElement), str(nNodeVal), str(nElementVal), str(nNodeVal+nElementVal)]) + ls]
    newLines += meshData
    newLines += nodeValName
    newLines += newNodeVal
    newLines += elementValName
    newLines += newElementVal
    return newLines

#
#  readCsvData
#    csvデータの読み込み a☓bのtableDataを読み込む
def readCsvData(fileName):
    #tab区切りorカンマ区切りのチェック
    #  1行読み込み
    f=open(fileName); line=f.readline(); f.close()
    if len(line.split("\t")) != 1:
        #tab区切り
        f=open(fileName); tabCont=f.readlines(); f.close()
        contents = []
        for line in tabCont:
            contents.append(",".join(line.split("\t")))
        cont = csv.reader(contents)
    else:
        #カンマ区切り
        cont = csv.reader(file(fileName))
    data = []
    for row in cont:
        if len(row) > 0:
            data.append(row)
    return data

#
#  isPrincipalStressInVtk
#    結果fileに主応力が追加されているかどうかチェック
def isPrincipalStressInVtk(lines):
    flag = False
    for line in lines:
        n = line.find("principalSigma1")
        if n >= 0:
            flag = True
            break
    return flag

#
#  isPrincipalStressVecInVtk
#    結果fileに主応力Vectorが追加されているかどうかチェック
def isPrincipalStressVecInVtk(lines):
    flag = False
    for line in lines:
        n = line.find("principalVectorSigma1")
        if n >= 0:
            flag = True
            break
    return flag

#
#  isStressMisesInVtk
#    結果fileにmises応力が追加されているかどうかチェック
def isStressMisesInVtk(lines):
    flag = False
    for line in lines:
        n = line.find("ndMisesStress")
        if n >= 0:
            flag = True
            break
    return flag

#
#  isPrincipalStrainInVtk
#    結果fileに主ひずみが追加されているかどうかチェック
def isPrincipalStrainInVtk(lines):
    flag = False
    for line in lines:
        n = line.find("principalEpsilon1")
        if n >= 0:
            flag = True
            break
    return flag

#
#  isPrincipalStrainVecInVtk
#    結果fileに主ひずみVectorが追加されているかどうかチェック
def isPrincipalStrainVecInVtk(lines):
    flag = False
    for line in lines:
        n = line.find("principalVectorEpsilon1")
        if n >= 0:
            flag = True
            break
    return flag

#
#  isStrainMisesInVtk
#    結果fileにmisesひずみが追加されているかどうかチェック
def isStrainMisesInVtk(lines):
    flag = False
    for line in lines:
        n = line.find("ndMisesStrain")
        if n >= 0:
            flag = True
            break
    return flag

#
#  isNodalStressInVtk
#    結果fileに応力tensorがあるかチェック
def isNodalStressInVtk(lines, stressTensor):
    flag = False
    for line in lines:
        n = line.find(stressTensor+" ")
        if n >= 0:
            flag = True
            break
    return flag

#
#  isNodalStrainInVtk
#    結果fileにひずみtensorがあるかチェック
def isNodalStrainInVtk(lines, strainTensor):
    flag = False
    for line in lines:
        n = line.find(strainTensor+" ")
        if n >= 0:
            flag = True
            break
    return flag

#
# isNameInVtk
#    結果file内の単語を検索する
def isNameInVtk(name, lines):
    flag = False
    for line in lines:
        n = line.find(name)
        if n >= 0:
            flag = True
            break
    return flag

#
#  addPrincipalStressDataInVtk
#    主応力を結果fileに追加
def addPrincipalStressDataInVtk(lines, addFlag, stressTensor):
    #追加するfield数を取得
    nField = sum([int(addFlag[0]), int(addFlag[1])*3, int(addFlag[2])*3,
                int(addFlag[6])])
    #FIELD数を修正
    i = 0
    while i < len(lines):
        n = lines[i].find("POINT_DATA ")
        if n >= 0:
            nn = -1
            while i < len(lines):
                nn = lines[i].find("FIELD ")
                if nn >= 0:
                    #field数を書き換え
                    words = lines[i].split()
                    words[2] = str(int(words[2]) + nField)   #fieldを追加
                    lines[i] = " ".join(words) + ls
                    break
                i += 1
            if nn >= 0:
                break
        i += 1
    i += 1
    insertP = i
    #主応力を計算
    addData = [[], [], [], [], [], [], [], []]
    while i < len(lines):
        n = lines[i].find(stressTensor+" ")
        if n >= 0:
            nNodes = int(lines[i].split()[2])
            i += 1
            ip = i
            while i < ip+nNodes:
                words = lines[i].split()
                stressList = []
                for word in words:
                    stressList.append(float(word))
                #主応力追加
                sig1=0.0; sig2=0.0; sig3=0.0
                sigv1=[0.0, 0.0, 0.0]; sigv2=[0.0, 0.0, 0.0]; sigv3=[0.0, 0.0, 0.0]
                if addFlag[1] == "1" or addFlag[2] == "1":
                    [sig1, sig2, sig3, sigv1, sigv2, sigv3] = calcPrincipalStress.principalStress(stressList)
                #mises応力を計算
                mises = 0.0
                if addFlag[0] == "1":
                    [sgxx, sgyy, sgzz, sgxy, sgyz, sgzx] = stressList
                    a = 0.5*(math.pow((sgxx-sgyy),2) + math.pow((sgyy-sgzz),2) + math.pow((sgzz-sgxx),2)) + 3.0*(sgxy*sgxy + sgyz*sgyz + sgzx*sgzx)
                    mises = math.pow(a, 0.5)
                #tresca応力を計算
                tresca = 0.0
                if addFlag[6] == "1":
                    if (sig1 == 0.0 and sig1 == 0.0 and sig3 == 0.0):
                        [sig1, sig2, sig3, sigv1, sigv2, sigv3] = calcPrincipalStress.principalStress(stressList)
                    tresca = sig1 - sig3
                addData[0].append(sig1)
                addData[1].append(sig2)
                addData[2].append(sig3)
                addData[3].append(sigv1)
                addData[4].append(sigv2)
                addData[5].append(sigv3)
                addData[6].append(mises)
                addData[7].append(tresca)
                i += 1
        if len(addData[0]) > 0:
            break
        i += 1
    #追加する行を作成
    addLines = []
    if addFlag[1] == "1":
        addLines += ["principalSigma1 1 " + str(nNodes) + " float" + ls]
        for val in addData[0]:
            addLines += [str(val) + ls]
        addLines += ["principalSigma2 1 " + str(nNodes) + " float" + ls]
        for val in addData[1]:
            addLines += [str(val) + ls]
        addLines += ["principalSigma3 1 " + str(nNodes) + " float" + ls]
        for val in addData[2]:
            addLines += [str(val) + ls]
    if addFlag[2] == "1":
        addLines += ["principalVectorSigma1 3 " + str(nNodes) + " float" + ls]
        for val in addData[3]:
            addLines += [" ".join([str(val[0]), str(val[1]), str(val[2])]) + ls]
        addLines += ["principalVectorSigma2 3 " + str(nNodes) + " float" + ls]
        for val in addData[4]:
            addLines += [" ".join([str(val[0]), str(val[1]), str(val[2])]) + ls]
        addLines += ["principalVectorSigma3 3 " + str(nNodes) + " float" + ls]
        for val in addData[5]:
            addLines += [" ".join([str(val[0]), str(val[1]), str(val[2])]) + ls]
    if addFlag[0] == "1":
        addLines += ["ndMisesStress 1 " + str(nNodes) + " float" + ls]
        for val in addData[6]:
            addLines += [str(val) + ls]
    if addFlag[6] == "1":
        addLines += ["ndTrescaStress 1 " + str(nNodes) + " float" + ls]
        for val in addData[7]:
            addLines += [str(val) + ls]
    #新しい行を追加
    lines = lines[:insertP] + addLines + lines[insertP:]
    return lines

#
#  addPrincipalStrainDataInVtk
#    主ひずみを結果fileに追加
def addPrincipalStrainDataInVtk(lines, addFlag, strainTensor):
    #追加するfield数を取得
    nFields = sum([int(addFlag[3]), int(addFlag[4])*3, int(addFlag[5])*3,
                    int(addFlag[7])])
    #FIELD数を修正
    i = 0
    while i < len(lines):
        n = lines[i].find("POINT_DATA ")
        if n >= 0:
            nn = -1
            while i < len(lines):
                nn = lines[i].find("FIELD ")
                if nn >= 0:
                    words = lines[i].split()
                    words[2] = str(int(words[2]) + nFields)
                    lines[i] = " ".join(words) + ls
                    break
                i += 1
            if nn >= 0:
                break
        i += 1
    i += 1
    insertP = i
    #主ひずみを計算
    addData = [[], [], [], [], [], [], [], []]
    while i < len(lines):
        n = lines[i].find(strainTensor+" ")
        if n >= 0:
            nNodes = int(lines[i].split()[2])
            i += 1
            ip = i
            while i < ip+nNodes:
                words = lines[i].split()
                strainList = []
                for word in words:
                    strainList.append(float(word))
                #ひずみtensor作成
                sList = [strainList[0], strainList[1], strainList[2], 
                            strainList[3]/2.0, strainList[4]/2.0, strainList[5]/2.0]
                #sList = [strainList[0], strainList[1], strainList[2], 
                #            strainList[3], strainList[4], strainList[5]]
                #主ひずみ取得
                eps1=0.0; eps2=0.0; eps3=0.0
                epsv1=[0.0, 0.0, 0.0]; epsv2=[0.0, 0.0, 0.0]; epsv3=[0.0, 0.0, 0.0]
                if addFlag[4] == "1" or addFlag[5] == "1":
                    [eps1, eps2, eps3, epsv1, epsv2, epsv3] = calcPrincipalStress.principalStress(sList)
                #misesひずみ計算
                mises = 0.0
                if addFlag[3] == "1":
                    [epxx, epyy, epzz, epxy, epyz, epzx] = sList
                    a = 0.5*(math.pow((epxx-epyy),2) + math.pow((epyy-epzz),2) + math.pow((epzz-epxx),2)) + 3.0*(epxy*epxy + epyz*epyz + epzx*epzx)
                    mises = math.pow(a, 0.5)
                #trescaひずみ計算
                tresca = 0.0
                if addFlag[7] == "1":
                    if (eps1 == 0.0 and eps2 == 0.0 and eps3 == 0.0):
                        [eps1, eps2, eps3, epsv1, epsv2, epsv3] = calcPrincipalStress.principalStress(sList)
                    tresca = eps1 - eps3
                #結果を保存
                addData[0].append(eps1)
                addData[1].append(eps2)
                addData[2].append(eps3)
                addData[3].append(epsv1)
                addData[4].append(epsv2)
                addData[5].append(epsv3)
                addData[6].append(mises)
                addData[7].append(tresca)
                i += 1
        if len(addData[0]) > 0:
            break
        i += 1
    #追加する行を作成
    addLines = []
    if addFlag[4] == "1":
        addLines += ["principalEpsilon1 1 " + str(nNodes) + " float" + ls]
        for val in addData[0]:
            addLines += [str(val) + ls]
        addLines += ["principalEpsilon2 1 " + str(nNodes) + " float" + ls]
        for val in addData[1]:
            addLines += [str(val) + ls]
        addLines += ["principalEpsilon3 1 " + str(nNodes) + " float" + ls]
        for val in addData[2]:
            addLines += [str(val) + ls]
    if addFlag[5] == "1":
        addLines += ["principalVectorEpsilon1 3 " + str(nNodes) + " float" + ls]
        for val in addData[3]:
            addLines += [" ".join([str(val[0]), str(val[1]), str(val[2])]) + ls]
        addLines += ["principalVectorEpsilon2 3 " + str(nNodes) + " float" + ls]
        for val in addData[4]:
            addLines += [" ".join([str(val[0]), str(val[1]), str(val[2])]) + ls]
        addLines += ["principalVectorEpsilon3 3 " + str(nNodes) + " float" + ls]
        for val in addData[5]:
            addLines += [" ".join([str(val[0]), str(val[1]), str(val[2])]) + ls]
    if addFlag[3] == "1":
        addLines += ["ndMisesStrain 1 " + str(nNodes) + " float" + ls]
        for val in addData[6]:
            addLines += [str(val) + ls]
    if addFlag[7] == "1":
        addLines += ["ndTrescaStrain 1 " + str(nNodes) + " float" + ls]
        for val in addData[7]:
            addLines += [str(val) + ls]
    #新しい行を追加
    lines = lines[:insertP] + addLines + lines[insertP:]
    return lines

#
#  addPrincipalStressData
#    結果データに主応力を追加する
def addPrincipalStressData(lines):
    #node数、element数を取得
    words = lines[0].split()
    nNodes = int(words[0])
    nElements = int(words[1])
    #NodalSTRESSを確認
    i = nNodes + nElements + 1
    nParas = lines[i].split()
    nVal = int(nParas[0])
    i += 1
    ip = i
    col = 1
    flag = "no"
    while i < ip + nVal:
        valName = deleteSp(lines[i]).split(",")[0]
        if valName == "NodalSTRESS":
            flag = "yes"
            break
        col += int(nParas[i-ip+1])
        i += 1
    if flag == "no":
        return lines

    #主応力を追加
    newLines = []
    #1行目を修正
    words = lines[0].split()
    words[2] = str(int(words[2]) + 3)
    words[4] = str(int(words[4]) + 3)
    lines[0] = " ".join(words) + ls
    #meshデータの次行を修正
    words = lines[ip-1].split()
    words[0] = str(int(words[0]) + 3)
    lines[ip-1] = " ".join(words) + " 1 1 1" + ls
    #変数を追加
    addLines =  ["maxSigma1, unit_unknown" + ls]
    addLines += ["midSigma2, unit_unknown" + ls]
    addLines += ["minSigma3, unit_unknown" + ls]
    newLines = lines[:ip+nVal] + addLines
    i = ip + nVal
    #while i < len(lines):
    n = 0
    while n < nNodes:
        stressWords = lines[i].split()[col:col+6]
        stressList = []
        for word in stressWords:
            stressList.append(float(word))
        [sig1, sig2, sig3] = calcPrincipalStress.principalStress(stressList)
        words = [str(sig1), str(sig2), str(sig3)]
        addWords = " ".join(words)
        #lines[i] = lines[i].split(ls)[0]+ " " + addWords + ls
        lines[i] = deleteLsOfLone(lines[i]) + " " + addWords + ls
        n += 1
        i += 1
    newLines += lines[ip+nVal:]
    return newLines

#  findNumberFromList
#    挟み撃ち法でlist中からnewNoを返す
#      入力:[[oldNo, newNo]...], No
#      戻り:newNo
def findNumberFromList(nodes, No):
    newNode = "0"
    if len(nodes) > 10:
        i = int(len(nodes)/2)
        node = nodes[i][0]
        if int(No) < int(node):
            newNodes = nodes[:i]
            newNode = findNumberFromList(newNodes, No)
            return newNode
        else:
            newNodes = nodes[i:]
            newNode = findNumberFromList(newNodes, No)
            return newNode
    else:
        for node in nodes:
            if node[0] == No:
                newNode = node[1]
                break
        return newNode


#
#  renumberMeshData
#    meshDataのみをrenumberする
def renumberMeshData(meshData, nNode, nElement):
    nodeData = meshData[:nNode]
    elementData = meshData[nNode:]
    #tmpデータ（list）を作成
    tmpNodes = []
    i = 1
    for line in nodeData:
        words = line.split()
        nodeNo = int(words[0])
        tmpNodes.append([nodeNo, i])
        nodeData[i-1] = str(i) + " " + " ".join(words[1:]) + ls
        i += 1
    maxNodeNo = max(tmpNodes)[0]
    nodeNoList = []
    #配列を準備
    i = 0
    while i < maxNodeNo+1:
        nodeNoList.append("")
        i += 1
    #list作成
    for [nodeNo, newNo] in tmpNodes:
        nodeNoList[nodeNo] = newNo
    #elementを修正
    i = 1
    for line in elementData:
        words = line.split()
        words[0] = str(i)
        newList = []
        for node in words[3:]:
            n = int(node)
            newNode = str(nodeNoList[n])
            newList.append(newNode)
        lineList = words[:3] + newList
        elementData[i-1] = " ".join(lineList) + ls
        i += 1
    newMeshData = nodeData + elementData
    return newMeshData

#
#  renumberRssultDataForParaview
#    node, elementのNoを修正する。
def renumberResultDataForParaview(lines):
    words = lines[0].split()
    nNode = int(words[0])
    nElem = int(words[1])
    #meshDataがrenumberされているかチェック
    if (nNode == int(lines[nNode].split()[0]) and
                nElem == int(lines[nNode+nElem].split()[0])):
        pass
    else:
        #meshDataをrenumberする
        meshLines = lines[1:1+nNode+nElem]
        meshLines = renumberMeshData(meshLines, nNode, nElem)
        lines = lines[:1] + meshLines + lines[1+nNode+nElem:]
    #nodeデータを修正
    i = nNode + nElem + 1
    words = lines[i].split()
    ir = i + int(words[0]) + 1
    i = 0
    while i < nNode:
        line = lines[i+ir]
        words = line.split()
        lines[i+ir] = str(i+1) + " " + " ".join(words[1:]) + ls
        i += 1
    #elementデータを修正
    ip = i + ir
    if ip < len(lines):
        nVal = int(lines[ip].split()[0])
        ir = ip + nVal + 1
        n = 0
        while n < nElem:
            words = lines[ir+n].split()
            lines[ir+n] = str(n+1) + " " + " ".join(words[1:]) + ls
            n += 1
    return lines

#
#  convertResultDataForParaview
#    UCD formatを旧のformatに変換する
def convertResultDataForParaview(lines):
    #3行skip
    lines = lines[3:]
    #node数、element数を取得
    words = lines[0].split()
    nNodes = int(words[0])
    nElements = int(words[1])
    #変換する全行を取得（要素座標直後の行を削除）
    i = nNodes + nElements
    newLines = lines[:i+1] + lines[i+2:]
    #出力変数の数を取得
    while i < len(lines):
        if lines[i][0] >= "A":
            break
        i += 1
    nVal = 0
    while i < len(lines):
        if "A" <= lines[i][0]:
            nVal += 1
        else:
            break
        i += 1
    #1行目を修正（節点の変数の数、要素の変数の数、モデルの変数の数）
    words = newLines[0].split()
    words += [str(nVal), "0", str(nVal)]
    newLines[0] = " ".join(words) + ls
    return newLines    

#
#  getEndPoint
#    !ENDを最後から検索してpointerを返す
def getEndPoint(lines):
    ip = 0
    i = len(lines) - 1
    while i > 0:
        line = lines[i]
        if line[:4] == "!END" or line[:4] == "!end" or line[:4] == "!End":
            ip = i
            break
        i -= 1
    return ip

#
#  getInsertPointInCnt
#    lineを追加する時の挿入場所を取得
#    keywordsの次の行No、存在しない場合はheader(!DLOAD等）の行Noが戻る
def getInsertPointInCnt(lines, header, keywords):
    ip = 0
    #keywordを検索
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[0] == "#":
            flag = 0
            for key in keywords:
                if line.find(key) >= 0:
                    flag = 1
                    break
            if flag == 1:
                i += 1
                if lines[i][0] == "#":
                    i += 1
                #while lines[i][0] == "#" and i < len(lines):
                #    i += 1
                ip =i
                break
        i += 1
    #keywordに該当したか？
    if ip == 0:
        #headerを検索
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len(header)] == header:
                ip = i
                break
            i += 1
    return ip

#
#  skipStepId
#    nStep分skipする
def skipStepId(lines, nStep):
    if nStep <= 0:
        return 0

    i = 0
    while i < len(lines):
        if lines[i][:len("!STEP")] == "!STEP":
            nStep -= 1
            if nStep <= 0:
                i += 1
                break
        i += 1
    return i

#
#  getBndInsertPointInCnt
#    境界条件の挿入行取得専用（nStepも検索する）
#    lineを追加する時の挿入場所を取得
#    keywordsの次の行No、存在しない場合はheader(!DLOAD等）の行Noが戻る
def getBndInsertPointInCnt(lines, header, keywords, nStep):
    if nStep <= 0:
        ip = getInsertPointInCnt(lines, header, keywords)
    else:
        ip = 0
        i = skipStepId(lines, nStep)
        while i < len(lines):
            if lines[i][0] == "!" or lines[i][0] == "#":
                ip = i
                break
            i += 1
    return ip 

#
#  remakeGRPID
#    header（!DLOAD等）のGRPIDを連番に修正する
def remakeGRPID(lines, header):
    if header == "!CLOAD" or header == "!DLOAD" or header == "!TEMPERATURE":
        grpId = 1
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[0] == "!":
                lineName = deleteSp(line).split(",")[0]
                if (lineName == "!CLOAD" or lineName == "!DLOAD"
                    or lineName == "!TEMPERATURE"):
                    lines[i] = abaqus2fistr.setValName(line, "GRPID="+str(grpId))
                    grpId += 1
            i += 1
    else:
        grpId = 1
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len(header)] == header:
                if deleteSp(lines[i]).split(",")[0] == header:
                    lines[i] = abaqus2fistr.setValName(line, "GRPID="+str(grpId))
                    grpId += 1
            i += 1
    return lines

#
#  getNextHeaderName
#    header名（!SOLVER等）を取得
def getNextHeaderName(lines, i, headerName, itemName):
    while i < len(lines):
        if lines[i][:len(headerName)] == headerName:
            if deleteSp(lines[i]).split(",")[0] == headerName:
                if itemName == "":
                    break
                else:
                    words = itemName.split("=")
                    if abaqus2fistr.getValName(lines[i], words[0]) == words[1]:
                        break
        i += 1
    return i

#
#  skipNextHeader
#
def skipNextHeader(lines, i):
    ans = -1
    while i < len(lines):
        if lines[i][0] == "!" or lines[i][0] == "#":
            ans = i
            break
        i += 1
    return ans

#
#  deleteHeaderName
#    指定したheaderを削除
def deleteHeaderName(lines, i, header, keyName):
    while i < len(lines):
        i = getNextHeaderName(lines, i, header, keyName)
        if i < len(lines):
            lines = delete1header(lines, i)
    return lines

#
#  delete1header
#    指定のheader位置からheader1ヶ分を削除（「#」または「!」まで
def delete1header(lines, i):
    ip = i
    i += 1
    while i < len(lines):
        if lines[i][0] == "#" or lines[i][0] == "!":
            break
        i += 1
    newLines = lines[:ip] + lines[i:]
    return newLines

#
#  getNextDloadGrpType
#    DLOADの値を取得
#    戻り値　grepTypes:[[grep, bndType, line, i], ...]
def getNextDloadGrpType(lines, i):
    ans = []
    flag = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!DLOAD")] == "!DLOAD":
            i += 1
            flag = 1
            grpType = []
            while lines[i][0] != "!" and i < len(lines):
                if lines[i][0] != "#":
                    line = deleteSp(lines[i])
                    words = line.split(",")
                    grpType += [[words[0], words[1], line, i]]
                i += 1
        else:
            i += 1
        if flag == 1:
            ans = grpType
            break
    return ans

#
#  getNoGroups
#    allGroupとselGroupからunselectのgroupを返す
def getNoGroups(allGroups, selGroups):
    noSels = []
    for allGroup in allGroups:
        flag = 0
        for selGroup in selGroups:
            if allGroup == selGroup:
                flag = 1
                break
        if flag == 0:
            noSels.append(allGroup)
    return noSels

#
#  getSolutionTypeFromCnt
#    solutionTypeを取得
def getSolutionTypeFromCnt():
    values = []
    lines = readFistrModelCnt()
    i = getNextHeaderName(lines, 0, "!SOLUTION", "")
    solType = abaqus2fistr.getValName(lines[i], "TYPE")
    if solType == "DYNAMIC":
        i = getNextHeaderName(lines, 0, "!DYNAMIC", "")
        i += 1
        if i < len(lines):
            words = deleteSp(lines[i]).split(",")
            if len(words) >= 2:
                if words[1] == "2":
                    solType = "DYNAMIC_F"
    elif solType == "HEAT":
        i = getNextHeaderName(lines, 0, "!HEAT", "")
        i += 1
        if i < len(lines):
            words = deleteSp(lines[i]).split(",")
            try:
                dt = float(words[0])
            except:
                dt = 0.0
            if dt > 0.0:
                solType = "HEAT_D"
    return solType

#
#  readFistrModelCnt
#    cntファイルを読み込み
def readFistrModelCnt():
    fileName = "FistrModel.cnt"
    if len(glob.glob(fileName)) == 0:
        return []
    f=open(fileName); lines=f.readlines(); f.close()
    return lines

#
#  getHeaderData
#    headerとそのdataを取得
def getHeaderData(lines):
    flag = 0
    headerData = []
    data = []
    header = ""
    for i in range(len(lines)):
        line = lines[i]
        if line[:1] == "!" and line[:2] != "!!":
            if header != "":
                headerData.append([header, data])
            header = lines[i]
            data = []
            flag = 1
        else:
            flag = 0
        if flag == 0:
            data.append(lines[i])
    headerData.append([header, data])
    return headerData

#
#  getHeaderNumData
#    headerとそのnumDataを取得
#    NODE:      [nodeNo, locx, locy, locz]形式で数値dataを取得
#    ELEMENT:   [elNo, node1, node2, node3, ...]形式で数値データを取得
#    EGROUP:    [ele1, ...]形式で数値データを取得
#    SGROUP:    [sur1, ...]形式で数値データを取得
#    NGROUP:    [node1, ...]形式で数値データを取得
def getHeaderNumData(lines):
    print _(u"メッシュデータを読み込み中...")
    headerData = getHeaderData(lines)
    i = 0
    for hData in headerData:
        header = hData[0]
        if header[:len("!NODE")] == "!NODE":
            print "  getting NODE..."
            dLines = hData[1]
            data = []
            for line in dLines:
                words = line.split(",")
                dataLine = [int(words[0]), float(words[1]), float(words[2]), float(words[3])]
                data.append(dataLine)
            headerData[i][1] = data
        elif header[:len("!ELEMENT")] == "!ELEMENT":
            typeName = getValName(header, "TYPE")
            print "  getting ELEMENT " + typeName + "..."
            dLines = hData[1]
            data = []
            for line in dLines:
                words = line.split(",")
                elData = []
                for word in words:
                    elData.append(int(word))
                data.append(elData)
            headerData[i][1] = data
        elif (header[:len("!EGROUP")] == "!EGROUP" or header[:len("!SGROUP")] == "!SGROUP"
            or header[:len("!NGROUP")] == "!NGROUP"):
            item = deleteSp(header).split(",")[0][1:]
            if item == "EGROUP":
                typeName = getValName(header, "EGRP")
            elif item == "SGROUP":
                typeName = getValName(header, "SGRP")
            elif item == "NGROUP":
                typeName = getValName(header, "NGRP")
            print "  getting " + item + " " + typeName + "..."
            dLines = hData[1]
            data = []
            for line in dLines:
                words = line.split(",")
                elData = []
                for word in words[:-1]:
                    elData.append(int(word))
                if skipSp(words[-1], 0) != len(words[-1]):
                    elData.append(int(words[-1]))
                data += elData
            headerData[i][1] = data
        i += 1
    return headerData

#
#  readUsingApplications
#    Applicationの設定を読み込む
def readUsingApplications():

    #空白をskip
    def skipSp(line, n):
        while n < len(line):
            if line[n] != " ":
                break
            n += 1
        return n

    #空白以外をskip
    def skipNsp(line, n):
        while n < len(line):
            if line[n] == " ":
                break
            n += 1
        return n

    #行末の空白を削除
    def deleteLastSp(command):
        i = len(command) - 1
        while i >= 0:
            if command[i] > " ":
                break
            i -= 1
        command = command[:i+1]
        return command

    #command（設定値）を取得
    def getCommand(line):
        n = skipSp(line, 0)
        n = skipNsp(line, n)
        n = skipSp(line, n)
        command = line[n:]
        if command != "":
            command = deleteLastSp(command)
        return command

    #commentを削除
    def deleteComment(lines):
        newLines = []
        for line in lines:
            newLine = ""
            for chara in line:
                if chara == "#":
                    newLine += ls
                    break
                else:
                    newLine += chara
            newLines.append(newLine)
        return newLines

    #usingAppの読み込み
    fileName = os.getenv("easyIstrUserPath") + os.sep + "data" + os.sep + "usingApp"
    f=open(fileName); lines=f.readlines(); f.close()
    if os.name == "nt":
        osName = "windows"
    else:
        osName = "linux"
    #コメント部を削除
    lines = deleteComment(lines)
    #変数の初期化
    office = ""; terminal = ""; fileManager = ""; editor = ""; frontIstrFolder = ""
    paraView = ""; revocap = ""; gnuplot = ""
    setFlag = [0, 0, 0, 0, 0, 0, 0, 0]
    flag = 0
    #keyWordと設定値を取得
    for line in lines:
        words = line.split()
        #osに従う内容を読み込むflagを設定
        if line[0] > " ":
            if words[0] == osName:
                flag = 1
            else:
                flag = 0
        #keywordと設定値読み込む
        if flag == 1 and line[0] == " " and len(words) > 0:
            if words[0] == "office":
                if setFlag[0] == 0:
                    office = getCommand(line)
                    setFlag[0] = 1
            elif words[0] == "terminal":
                if setFlag[1] == 0:
                    terminal = getCommand(line)
                    setFlag[1] = 1
            elif words[0] == "fileManager":
                if setFlag[2] == 0:
                    fileManager = getCommand(line)
                    setFlag[2] = 1
            elif words[0] == "editor":
                if setFlag[3] == 0:
                    editor = getCommand(line)
                    setFlag[3] = 1
            elif words[0] == "frontIstrFolder":
                if setFlag[4] == 0:
                    frontIstrFolder = getCommand(line)
                    setFlag[4] = 1
            elif words[0] == "paraView":
                if setFlag[5] == 0:
                    paraView = getCommand(line)
                    setFlag[5] = 1
            elif words[0] == "REVOCAP":
                if setFlag[6] == 0:
                    revocap = getCommand(line)
                    setFlag[6] = 1
            elif words[0] == "gnuplot":
                if setFlag[7] == 0:
                    gnuplot = getCommand(line)
                    setFlag[7] = 1
        if sum(setFlag) >= 8:
            break
    return [office, terminal, fileManager, editor, frontIstrFolder, paraView, revocap, gnuplot]

#
#  deleteSp
#    空白とCRを削除して返す
def deleteSp(line):
    newLine = ""
    i = 0
    for chara in line:
        if chara > " ":
            newLine += chara
        else:
            if i >= 1 and chara == " ":
                if line[i-1] == " " or line[i-1] == ",":
                    pass
                else:
                    newLine += chara
        i += 1 
    return newLine

#
#  getValName
#    lineからvalName(TYPEなど）を取得
def getValName(line, valName):
    words = line.split(",")
    name = ""
    for word in words:
        # 「TYPE=」などを検索
        if word.find(valName+"=") >= 0:
            name = word.split("=")[1]
            break
    newName = ""
    for chara in name:
        if chara > " ":
            newName += chara
    return newName

#
#  deleteValName
#    line内から変数名を削除する
def deleteValName(line, delName):
    words = deleteSp(line).split(",")
    newWords = []
    for word in words:
        valSet = word.split("=")
        if len(valSet) == 2 and valSet[0] == delName:
            pass
        else:
            newWords.append(word)
    newLine = ", ".join(newWords) + ls
    return newLine

#
#  deleteSameData
#    list内から同じdataを削除する
def deleteSameData(dataList):
    if len(dataList) == 0:
        return dataList
    dataList.sort()
    bData = dataList[0]
    newDataList = [bData]
    for data in dataList:
        if data != bData:
            newDataList.append(data)
            bData = data
    return newDataList    

#
#  runAppBG
#    appをバックグラウンドで実行
def runAppBG(comm):
    if os.name == "nt":
        #windowsの場合
        comm = "start /b cmd /c " + comm
    else:
        #linuxの場合
        comm = comm + " &"
    print comm
    os.system(comm)

#
#  delFiles
#    fileを削除
def removes(filePattern):
    delNames = glob.glob(filePattern)
    if len(delNames) > 0:
        for delName in delNames:
            os.remove(delName)
    return

#
#  deleteLsOfLine
#    行末ゴードを削除
def deleteLsOfLine(line):
    if line[-2:] == "\r\n":
        newLine = line[:-2]
    else:
        newLine = line[:-1]
    return newLine

#
#  skipChara
#    空白以下をskipする
def skipChara(cont, p):
    while cont[p] > " " and p < len(cont):
        p += 1
    return p

#
#  skipSp
#    空白以下をskipする
def skipSp(cont, p):
    while p < len(cont) and cont[p] <= " ":
        p += 1
    return p

